@@ -84,9 +84,8 @@ class S3Storage(Storage):
             entity.cw_edited.edited_attribute(attr, None)
             self.entity_deleted(entity, attr)
         else:
-            key = self.get_s3_key(entity, attr)
-            if key is None:
-                key = self.new_s3_key(entity, attr)
+            oldkey = self.get_s3_key(entity, attr)
+            key = self.new_s3_key(entity, attr)
             # save S3 key
             entity.cw_edited.edited_attribute(
                 attr, Binary(key.encode('utf-8')))
@@ -108,6 +107,9 @@ class S3Storage(Storage):
                 (self, key, entity.eid, attr))
             self.info('Temporary uploaded %s.%s (%s/%s) to S3',
                       entity.eid, attr, self.bucket, suffixed_key)
+            if oldkey is not None and oldkey != key:
+                # remove unneeded old file
+                self.delay_deletion(entity, attr, oldkey)
         return binary
 
     def get_upload_extra_args(self, _entity, _attr, _key):
@@ -123,11 +125,14 @@ class S3Storage(Storage):
 
     def entity_deleted(self, entity, attr):
         """an entity using this storage for attr has been deleted"""
+        key = self.get_s3_key(entity, attr)
+        if key is None:
+            # no key to remove
+            return
+        self.delay_deletion(entity, attr, key)
+
+    def delay_deletion(self, entity, attr, key):
         if entity._cw.repo.config['s3-auto-delete']:
-            key = self.get_s3_key(entity, attr)
-            if key is None:
-                # no key to remove
-                return
             # delete key in a post commit event
             S3DeleteFileOp.get_instance(entity._cw).add_data(
                 (self, key, entity.eid, attr))
